﻿using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;

namespace Gauss.Isp.Attributes
{
    /// <summary>
    /// This is an extension of the DisplayNameAttribute class, supporting
    /// localized display name strings.
    /// </summary>
    public sealed class LocalizedDisplayNameAttribute : DisplayNameAttribute
    {
        /// <summary>
        /// This is the current resource manager which is used to convert the
        /// display resource ID to the localized version.
        /// </summary>
        private static global::System.Resources.ResourceManager _resourceManager;

        /// <summary>
        /// This is the localized string that represents the given name Id.
        /// </summary>
        private string _localizedNameString;

        /// <summary>
        /// This returns the resource manager used to get the localized string,
        /// and it caches the value in _resourceManager.
        /// </summary>
        internal static global::System.Resources.ResourceManager ResourceManager
        {
            get
            {
                if (_resourceManager == null)
                {
                    _resourceManager = new global::System.Resources.ResourceManager("Gauss.Isp.Resources", typeof(Resources).Assembly);
                }
                return _resourceManager;
            }
        }

        /// <summary>
        /// This creates a display name by converting the nameId to the correct
        /// localized string.
        /// </summary>
        /// <param name="nameId">The resource ID of the name in the resources file.</param>
        public LocalizedDisplayNameAttribute(string nameId)
            : base(nameId)
        {
            this._localizedNameString = ResourceManager.GetString(nameId);
        }

        /// <summary>
        /// Returns the localized string for the given name ID.
        /// </summary>
        public override string DisplayName
        {
            get
            {
                return _localizedNameString;
            }
        }

    }
}
